@@ -8,7 +8,10 @@
 #include <Python.h>
 #include <windows.h>
 
-
+PyObject * NoSuchProcess(void);
+PyObject * AccessDenied(void);
+HANDLE handle_from_pid_waccess(DWORD pid, DWORD dwDesiredAccess);
+HANDLE handle_from_pid(DWORD pid);
 PVOID GetPebAddress(HANDLE ProcessHandle);
 HANDLE handle_from_pid(DWORD pid);
 BOOL is_running(HANDLE hProcess);
